import QtQuick 2.5
import com.zuragon.ViCANdo 1.0
import QtQuick.Extras 1.4
import QtQuick.Controls 1.4
import QtQuick.Layouts 1.3
import QtQuick.Controls.Styles 1.4


Rectangle {
        width: 900
        height: 300
        color: "lightgrey"
        id: qml_send_can_example
        visible: true

        Text{
            id: title
            text: " ViCANdo QML example for sending CAN message "
            font.family: "Helvetica"
            anchors.topMargin: 15
            anchors.horizontalCenter: parent.horizontalCenter
            font.pointSize: 18
        }

        Text{
            id: instruction
            color: "blue"
            text: "Press 'record' button first, then configure the message to be sent and click send button to send. "
            font.family: "Helvetica"
            anchors.top: title.bottom
            anchors.topMargin: 5
            anchors.horizontalCenter: parent.horizontalCenter
            font.pointSize: 12
        }


        Rectangle {
            id: line_separator_1
            height: 2
            color: "#000000"
            anchors.top: instruction.bottom
            anchors.topMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }

        GroupBox {
            id: group_box_config
            title: "CAN message configuration"
            anchors.top: instruction.bottom
            anchors.topMargin: 15
            anchors.left: parent.left
            anchors.leftMargin: 5

            RowLayout {
                id: first_row
                Text{
                    id: tx__id_label
                    text: "Msg ID: 0x"
                    font.family: "Helvetica"
                }
                TextField{
                    id: tx_id
                    text: "501"
                    font.family: "Helvetica"
                }

                ExclusiveGroup { id: id_type_group }
                        RadioButton {
                            id: standard_id
                            text: "Standard ID"
                            checked: true
                            exclusiveGroup: id_type_group
                        }
                        RadioButton {
                            id: extended_id
                            text: "Extended ID"
                            exclusiveGroup: id_type_group
                        }

            }

            RowLayout {
                id: second_row
                anchors.top: first_row.bottom
                anchors.topMargin: 5
                Text{
                    id: tx_dlc_label
                    text: "DLC:"
                    font.family: "Helvetica"

                }
                TextField{
                    id: tx_dlc
                    text: "8"
                    font.family: "Helvetica"
                }
            }

            RowLayout{
                id: third_row
                anchors.top: second_row.bottom
                anchors.topMargin: 5
                Text{
                    id: data_label
                    text: "Data(hex):"
                    font.family: "Helvetica"
                }
            }
            RowLayout{
                id: fourth_row
                anchors.top: third_row.bottom
                anchors.topMargin: 5
                Text{
                    id: byte0_label
                    text: "        BYTE0           "
                    font.family: "Helvetica"
                }
                Text{
                    id: byte1_label
                    text: "        BYTE1           "
                    font.family: "Helvetica"
                }
                Text{
                    id: byte2_label
                    text: "        BYTE2           "
                    font.family: "Helvetica"
                }
                Text{
                    id: byte3_label
                    text: "        BYTE3           "
                    font.family: "Helvetica"
                }
                Text{
                    id: byte4_label
                    text: "        BYTE4           "
                    font.family: "Helvetica"
                }
                Text{
                    id: byte5_label
                    text: "        BYTE5           "
                    font.family: "Helvetica"
                }
                Text{
                    id: byte6_label
                    text: "        BYTE6           "
                    font.family: "Helvetica"
                }
                Text{
                    id: byte7_label
                    text: "        BYTE7           "
                    font.family: "Helvetica"
                }
            }
            RowLayout{
                id: fifth_row
                anchors.top: fourth_row.bottom
                anchors.topMargin: 5
                TextField{
                    id: byte0
                    text: "11"
                    font.family: "Helvetica"
                }
                TextField{
                    id: byte1
                    text: "22"
                    font.family: "Helvetica"
                }
                TextField{
                    id: byte2
                    text: "33"
                    font.family: "Helvetica"
                }
                TextField{
                    id: byte3
                    text: "11"
                    font.family: "Helvetica"
                }
                TextField{
                    id: byte4
                    text: "11"
                    font.family: "Helvetica"
                }
                TextField{
                    id: byte5
                    text: "11"
                    font.family: "Helvetica"
                }
                TextField{
                    id: byte6
                    text: "11"
                    font.family: "Helvetica"
                }
                TextField{
                    id: byte7
                    text: "11"
                    font.family: "Helvetica"
                }

            }
            RowLayout{
                id: sixth_row
                anchors.top: fifth_row.bottom
                anchors.topMargin: 5

                Text{
                    id: tx_interval_label
                    text: " Transmit every "
                    font.family: "Helvetica"
                }

                TextField{
                    id: tx_interval
                    text: "1000"
                    font.family: "Helvetica"
                }

                Text{
                    id: tx_interval_ms_label
                    text: "ms"
                    font.family: "Helvetica"
                }
                Button {
                    id: send_button
                    text: "Send"
                    onClicked: {
                        timer.start()
                    }
                }
                Button {
                    id: stop_button
                    text: "Stop"
                    onClicked: {
                        timer.stop()
                    }
                }
                Text{
                    id: status_label
                    color: timer.running ? "green":"red"
                    text: timer.running ? " sending..." : " stopped"
                }
            }

        }

        function getCANData() {
            var dlen = parseInt(tx_dlc.text)
            var data = []
            var values = [ byte0.text,
                           byte1.text,
                           byte2.text,
                           byte3.text,
                           byte4.text,
                           byte5.text,
                           byte6.text,
                           byte7.text ]

            for ( var i = 0; i < dlen; ++i ) {
                data[i] = parseInt(values[i], 16)
            }
            return data;
        }

        Timer {
            id: timer
            interval: parseInt(tx_interval.text)
            running: false
            repeat: true
            onTriggered:{

                var id = parseInt(tx_id.text, 16)
                var data = getCANData()

                // Send the message every selected ms
                if( standard_id.checked){
                    project.findObject("can0").sendStandard(id, data)
                }else{
                    project.findObject("can0").sendExtended(id, data)
                }
                project.log("Send msg --> id(dec): " + id + "   data(dec):  [" + data + "]")

            }
        }

        // For listenning to CAN messages received by this source "can0"
        SourceEventListener {
            id: listen_to_can_msg
            source: project.findObject("can0") // can0 is the first CAN source
            //dbc: project.findObject("object_name_of_my_dbc")
            // filter: ["msgName1","msgName2"]
            onFrameDataReceived: {
                project.log("Source can0 Rx msg id: " + _data.id + " dlc: " + _data.dlc + " data: " + _data.data)
            }
        }

        // For listenning to ViCANdo project state change.
        ProjectStateEventListener {
            id: project_state_listener

            onIdle: {
               project.log("Project Idle " + " at time " + time_in_us + " us")
            }

            onPreparing: {
               project.log("Project Preparing" + " at time " + time_in_us + " us")
            }

            onArmed: {
                project.log("Project Armed " + " at time " + time_in_us + " us")
            }

            onRecording: {
                project.log("Project Recording " + " at time " + time_in_us + " us")
            }

            onPlaying: {
                project.log("Project Playing " + " at time " + time_in_us + " us")
            }
        }




}
